@@ -4,7 +4,7 @@ from os import path
 
 from javalang.tree import *
 
-from javacoder.utils.converter import get_first_element
+from javacoder.utils.converter import get_first_element, parse_expression, convert_literal_value
 from javacoder.utils.loader import load_content
 from javacoder.core.java_generator import JavaCodeGenerator
 from javalang import parse
@@ -75,11 +75,14 @@ class ObjectOperator:
             raise Exception("file_name_cannot_be_none")
 
     @pre_notify_plugins
-    def preview_class(self):
+    def preview_class(self, display=True):
         self.format()
         gen = JavaCodeGenerator()
         gen.generate(self.class_tree)
-        info('previewed class: \n---{}---\n{}\n---end---'.format(self.file_name, gen.get_code()))
+        if not gen.success:
+            debug(f'generate source failed, path: {self.file_path}')
+        if display:
+            info('previewed class: \n---{}---\n{}\n---end---'.format(self.file_name, gen.get_code()))
 
     @pre_notify_plugins
     def save(self):
@@ -112,16 +115,19 @@ class ObjectOperator:
         if self.class_cursor:
             if not self.class_cursor.annotations:
                 self.class_cursor.annotations = []
-            self.class_cursor.annotations.append(
-                Annotation(name=annotation_name,
-                           element=[
-                               ElementValuePair(name=key,
-                                                value=Literal(value=value) if type(value) in (
-                                                    str, int, bool) else value)
-                               for
-                               key, value
-                               in
-                               kwargs.items()]))
+            if len(kwargs) == 1 and 'value' in kwargs.keys():
+                self.class_cursor.annotations.append(
+                    Annotation(name=annotation_name,
+                               element=Literal(value=convert_literal_value(kwargs['value']))
+                               if type(kwargs['value']) in (int,str,bool) else kwargs['value'])
+                )
+            else:
+                self.class_cursor.annotations.append(
+                    Annotation(name=annotation_name,
+                               element=[
+                                   ElementValuePair(name=key,
+                                                    value=Literal(value=convert_literal_value(value)))
+                                   for key, value in kwargs.items()]))
         else:
             error("failed add annotation only if set class_cursor before")
 
@@ -316,7 +322,10 @@ class ClassOperator(ObjectOperator):
         else:
             self.file_content = load_content(class_path)
             self.class_tree = parse.parse(self.file_content)
-            self.class_cursor = self.class_tree.types[0]
+            if len(self.class_tree.types) > 0:
+                self.class_cursor = self.class_tree.types[0]
+            else:
+                warning(f'java file [{class_path}] is empty.')
 
     def __create_default_class(self, name: str = '', file_path: str = ''):
         self.class_tree = CompilationUnit()
@@ -348,7 +357,8 @@ class ClassOperator(ObjectOperator):
             return fields
 
     def format(self):
-        self.class_cursor.body = sorted(self.class_cursor.body, key=get_class_member_type_order)
+        if self.class_cursor and self.class_cursor.body:
+            self.class_cursor.body = sorted(self.class_cursor.body, key=get_class_member_type_order)
 
     def add_private_string(self, field_name, field_value=''):
         self.get_field_cursor().add_private_string_field(field_name, field_value)
@@ -395,12 +405,15 @@ class ClassMemberOperator:
             self.class_operator.class_cursor.type_parameters = []
         self.member_cursor = self.class_operator.class_cursor.body
 
-    def __get_field(self, field_name):
+    def _get_field(self, field_name):
         for item in self.member_cursor:
             if type(item) == FieldDeclaration and field_name in [d.name for d in item.declarators]:
                 return item
         return None
 
+    def add(self, expression):
+        self.member_cursor.append(parse_expression(expression))
+
 
 class MethodOperator(ClassMemberOperator):
     def __init__(self, class_operator):
@@ -422,7 +435,7 @@ class MethodOperator(ClassMemberOperator):
     def add_getter(self, field_name: str):
         modifier = set()
         modifier.add('public')
-        field = self.__get_field(field_name)
+        field = self._get_field(field_name)
         if field:
             self.add_method(modifier, field.type, 'get' + field_name[0].upper() + field_name[1:],
                             body=[ReturnStatement(expression=MemberReference(member=field_name))])
@@ -430,7 +443,7 @@ class MethodOperator(ClassMemberOperator):
     def add_setter(self, field_name: str):
         modifier = set()
         modifier.add('public')
-        field = self.__get_field(field_name)
+        field = self._get_field(field_name)
         if field:
             self.add_method(modifier, None, 'set' + field_name[0].upper() + field_name[1:],
                             parameters=[FormalParameter(type=field.type, name=field_name)],
@@ -468,20 +481,20 @@ class FieldOperator(ClassMemberOperator):
         modifier = set()
         modifier.add('private')
         self.add_field(modifier, ReferenceType(name="String"), [
-            VariableDeclarator(name=field_name, initializer=Literal(value=value) if len(value) > 0 else None)])
+            VariableDeclarator(name=field_name,
+                               initializer=Literal(value=convert_literal_value(value)) if len(value) > 0 else None)])
 
     def add_field_annotation(self, field_name, annotation_name, **annotation_kwargs):
         field = self.get_field(field_name)
         if not field.annotations:
             field.annotations = []
         field.annotations.append(Annotation(name=annotation_name,
-                                            element=[ElementValuePair(name=key, value=Literal(value=value)) for
-                                                     key, value
-                                                     in
-                                                     annotation_kwargs.items()]))
+                                            element=[ElementValuePair(name=key,
+                                                                      value=Literal(value=convert_literal_value(value)))
+                                                     for key, value in annotation_kwargs.items()]))
 
     def get_field(self, field_name):
-        return self.__get_field(field_name)
+        return self._get_field(field_name)
 
     def add_field(self, modifiers, field_type: ReferenceType, declarators: list):
         self.member_cursor.append(FieldDeclaration(modifiers=modifiers, type=field_type, declarators=declarators))
